import requests
from bs4 import BeautifulSoup
import os
import logging
from urllib.parse import urljoin, urlparse
import hashlib
from pathlib import Path

logger = logging.getLogger(__name__)

# Define the directory to save images within the instance folder
IMAGE_SAVE_DIR = Path(__file__).resolve().parent.parent / "instance" / "article_images"
IMAGE_SAVE_DIR.mkdir(parents=True, exist_ok=True) # Ensure directory exists

def get_best_image_url(html_content: str, base_url: str) -> str | None:
    """Parses HTML to find the best image URL (og:image, twitter:image, or first large img)."""
    soup = BeautifulSoup(html_content, 'html.parser')

    # Prioritize Open Graph image
    og_image = soup.find('meta', property='og:image')
    if og_image and og_image.get('content'):
        img_url = og_image['content']
        return urljoin(base_url, img_url) # Make URL absolute

    # Fallback to Twitter image
    twitter_image = soup.find('meta', attrs={'name': 'twitter:image'})
    if twitter_image and twitter_image.get('content'):
        img_url = twitter_image['content']
        return urljoin(base_url, img_url)

    # Fallback to first image in main content (simple approach)
    # More complex logic could check image size, etc.
    main_content = soup.find('article') or soup.find('main')
    if main_content:
        first_img = main_content.find('img')
        if first_img and first_img.get('src'):
            # Ignore tiny images or data URIs often used for spacers/icons
            src = first_img['src']
            if not src.startswith('data:image') and not ('1x1' in src or 'spacer' in src):
                 # Basic check for width/height attributes, prefer larger ones if specified
                 width = first_img.get('width', '0')
                 height = first_img.get('height', '0')
                 try:
                     # Check if dimensions suggest it's not a tiny icon
                     if int(width.replace('px','')) > 50 and int(height.replace('px','')) > 50:
                         img_url = first_img['src']
                         return urljoin(base_url, img_url)
                 except ValueError: # Handle non-pixel width/height like 'auto'
                      img_url = first_img['src']
                      return urljoin(base_url, img_url)

    return None

def download_and_save_image(image_url: str, base_filename_hint: str) -> str | None:
    """Downloads an image from a URL and saves it locally, returning the filename."""
    if not image_url:
        return None

    headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'}
    filename = None # Initialize filename
    try:
        response = requests.get(image_url, headers=headers, timeout=15, stream=True)
        response.raise_for_status()

        # Get file extension from URL or Content-Type
        content_type = response.headers.get('content-type')
        extension = '.jpg' # Default extension
        if content_type:
            if 'jpeg' in content_type: extension = '.jpg'
            elif 'png' in content_type: extension = '.png'
            elif 'gif' in content_type: extension = '.gif'
            elif 'webp' in content_type: extension = '.webp'
        else:
            # Try getting extension from URL path
            parsed_path = urlparse(image_url).path
            file_ext = os.path.splitext(parsed_path)[1]
            if file_ext.lower() in ['.jpg', '.jpeg', '.png', '.gif', '.webp']:
                extension = file_ext.lower()

        # Create a unique filename using hash of URL + hint + extension
        # Use hint (like article id) to make filename somewhat predictable
        filename_base = hashlib.sha1(image_url.encode()).hexdigest()[:10] # Short hash
        filename = f"{base_filename_hint}_{filename_base}{extension}"
        save_path = IMAGE_SAVE_DIR / filename

        # Save the image
        with open(save_path, 'wb') as f:
            for chunk in response.iter_content(1024):
                f.write(chunk)

        # logger.info(f"Successfully downloaded and saved image to {save_path}") # Removed, too verbose
        return filename # Return just the filename

    except requests.exceptions.RequestException as e:
        logger.error(f"Failed to download image {image_url}: {e}")
        return None
    except Exception as e:
        # Use the filename variable captured before the error, if available
        save_path_str = str(IMAGE_SAVE_DIR / filename) if filename else "unknown path"
        logger.error(f"Failed to save image {image_url} to {save_path_str}: {e}", exc_info=True)
        return None

def fetch_extract_save_image(article_url: str, article_id: int) -> str | None:
    """Fetches article HTML, extracts best image, downloads and saves it."""
    if not article_url:
        return None

    headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'}
    try:
        response = requests.get(article_url, headers=headers, timeout=10)
        response.raise_for_status()

        image_url = get_best_image_url(response.text, article_url)
        if image_url:
            # Use article_id as hint for filename
            filename = download_and_save_image(image_url, f"article_{article_id}")
            return filename
        else:
            logger.info(f"No suitable image URL found for article {article_id} at {article_url}")
            return None

    except requests.exceptions.RequestException as e:
        logger.error(f"Failed to fetch article HTML {article_url} for image extraction: {e}")
        return None
    except Exception as e:
        logger.error(f"Error during image extraction process for {article_url}: {e}", exc_info=True)
        return None
